/**
 * Cassandra loader script
 * @return {Cassandra}
 */

function cassandraLoader () {}

/**
 * Default keyspace
 * @type {string}
 */
cassandraLoader.prototype.defaultcluster = '';

/**
 * Default keyspace
 * @type {string}
 */
cassandraLoader.prototype.defaultkeyspace = '';

/**
 * References to adaptors (a new adaptor is
 * created for each unique keyspace)
 * @type {{}}
 */
cassandraLoader.prototype.adaptors = {};

/**
 * Load
 * @param {object} cluster
 * @param {string} keyspace
 * @param {function} cb
 */
cassandraLoader.prototype.load = function(cluster, keyspace, cb) {

	if (!cluster && !keyspace) {
		throw new Error(
			'Cassandra missing config. Define hosts/user/pass/keyspace in the configuration file'
		);
	}

	this.defaultcluster		= cluster;
	this.defaultkeyspace    = keyspace;

	this.link(keyspace);

	cb();
};

/**
 * Return a singleton for the adaptor based on database (there will be a
 * different connection pool and adaptor object for each db name used)
 * @param {string} db
 * @return {object}
 */
cassandraLoader.prototype.link = function(keyspace) {

	keyspace = keyspace || this.defaultkeyspace;

	if (!this.adaptors || !this.adaptors[keyspace]) {

		this.adaptors[keyspace] = this.kernel.include(this.kernel.kerneldir + 'adaptors/cassandra').singleton(
			 keyspace
			,this.defaultcluster
			,keyspace
		);

		/**
		 * Set consistency level for this keyspace
		 * @type {string}
		 */
		this.adaptors[keyspace].CONSISTENCY_INSERT	= this.adaptors[keyspace].CONSISTENCY_LOCAL_QUORUM;
		this.adaptors[keyspace].CONSISTENCY_SELECT	= this.adaptors[keyspace].CONSISTENCY_LOCAL_QUORUM;
		this.adaptors[keyspace].CONSISTENCY_UPDATE	= this.adaptors[keyspace].CONSISTENCY_LOCAL_QUORUM;
		this.adaptors[keyspace].CONSISTENCY_DELETE	= this.adaptors[keyspace].CONSISTENCY_LOCAL_QUORUM;
		this.adaptors[keyspace].CONSISTENCY_BATCH	= this.adaptors[keyspace].CONSISTENCY_LOCAL_QUORUM;
	}

	return this.adaptors[keyspace];
};

/**
 * Gather up statuses from all open adaptors
 * @return {object}
 */
cassandraLoader.prototype.status = function() {
	var merge = {};
	for (var keyspace in this.adaptors) {
		merge[keyspace] = this.adaptors[keyspace].status;
	}
	return merge;
};

/**
 * Unload all adaptors (prepare for shutdown)
 * @param {function} cb
 */
cassandraLoader.prototype.unload = function(cb) {
	noloaded = Object.keys(this.adaptors).length;
	if (!noloaded) {
		cb(null, 'pool not created');
	} else {
		for (var keyspace in this.adaptors) {
			this.adaptors[keyspace].shutdown(function() {
				if (!--noloaded) {
					cb(null, 'pool drained');
				}
			}.bind(this));
		}
	}
};

/**
 * Export
 */
exports = module.exports = cassandraLoader;