/******************************************************************************
 * Lemonade JS
 * https://lemonadejs.com
 *
 * ----
 *
 * Solr link loader script
 *
 *****************************************************************************/

function solrLoader () {}

/**
 * Master config
 * @type {{}}
 */
solrLoader.prototype.master = {};

/**
 * Slave config
 * @type {{}}
 */
solrLoader.prototype.slave = {};

/**
 * Default core
 * @type {string}
 */
solrLoader.prototype.defaultcore = '';

/**
 * References to adaptors (a new adaptor is
 * created for each unique core)
 * @type {{}}
 */
solrLoader.prototype.adaptors = {};

/**
 * Try to execute a simple query on both master and slave
 * @param {object} master
 * @param {object} slave
 * @param {string} core
 * @param {function} cb
 */
solrLoader.prototype.load = function(master, slave, core, cb) {
    if (!master && !slave) {
        throw new Error(
            'Solr invalid config. Define host/user/pass/port ' +
            'in the configuration file'
        );
    }
    if (master) {
        this.master = master;
    }
    if (slave) {
        this.slave = slave;
    }
    if (!core) {
        throw new Error(
            'Solr invalid config. Missing default core'
        );
    }
    this.defaultcore = core;
    /**
     * Prepare a cb to be ran after executing a query on the master
     * and one on the slave
     */
    var notoload = 2, retmsg = '', precb = function(err, msg) {
        if (err) {
            cb(err);
        } else {
            if (msg) {
                retmsg += msg + ',';
            }
            if (!--notoload) {
                cb(null, '(' + retmsg.substring(0, retmsg.length - 1) + ') searcher up');
            }
        }
    }
    /**
     * Build a test query
     */
   ,q = this.link().query();
    q . q('test');
    q . rows(1);
    /**
     * Test master client
     */
    this.link()._master.search(q, function(err) {
        precb(err, 'master');
    });
    /**
     * If we got a non-replication setup don't bother
     */
    if (slave && master !== slave) {
        /**
         * Test slave client
         */
        this.link().search(q, function(err) {
            precb(err, 'slave');
        });
    } else {
        precb(null, '');
    }
};

/**
 * Return a singleton for the adaptor based on the solr core (there will be a
 * different connection pool and adaptor object for each core name used)
 * @param {string} core
 * @return {object}
 */
solrLoader.prototype.link = function(core) {
    core = core || this.defaultcore;
    if (!this.adaptors[core]) {
        this.adaptors[core] = this.kernel.include(this.kernel.kerneldir + 'adaptors/solr').singleton(core,
            core, this.master, this.slave
        );
    }
    return this.adaptors[core];
};

/**
 * Gather up statuses from all open adaptors
 * @return {object}
 */
solrLoader.prototype.status = function() {
    var merge = {};
    for (var core in this.adaptors) {
        merge[core] = this.adaptors[core].status;
    }
    return merge;
};

/**
 * Unload all adaptors (prepare for shutdown)
 * @param {function} cb
 */
solrLoader.prototype.unload = function(cb) {
    noloaded = Object.keys(this.adaptors).length;
    if (!noloaded) {
        cb(null, 'not started');
    } else {
        for (var core in this.adaptors) {
            this.adaptors[core].shutdown(function() {
                if (!--noloaded) {
                    cb(null, 'stopped');
                }
            }.bind(this));
        }
    }
};

/**
 * Export
 */
exports = module.exports = solrLoader;