#!/usr/bin/env node

var   fs     = require('fs')
    , exec   = require('child_process').exec
    , cwd    = process.cwd()
    , path   = require('path')
    , kernel = require('../kernel/kernel.js')
    , usage  =

        '\n' +
        'Usage\t: lemonade [options] config-file.js\n' +
        'Options\t: \n' +
        '   --help,-h'           + '\t\tthis message\n' +
        '   --version,-v'        + '\t\tlemonade version\n' +
        '   --watch-changes,-w'  + '\twatch for changes in the app dir;\n' +
        '\t\t\tgracefully restart when a file changes\n';

if (process.argv[2] === 'install') {
    exec(
        'cp -rv ' + __dirname + '/../app/* ' + cwd,
        function (error, stdout, stderr) {
            if (!error) {
                console.log('Install completed');
            } else {
                console.log(error, stderr);
            }
        }
    );
} else {
    /**
     * Run options
     */
    for (var key in process.argv) {
        var option = process.argv[key];
        switch (option) {
            case '--help' :
            case '-h' :
                console.log(usage);
                process.exit(0);
                break;
            case '--version' :
            case '-v' :
                console.log(kernel.prototype.version);
                process.exit(0);
                break;
            case '--watch-changes' :
            case '-w' :
                var watch = true;
                break;
        }
    }
    var configfile = process.argv.pop();
    /**
     * Check the config file and chdir into it
     */
    if (fs.existsSync(cwd + '/' + configfile)) {
        var config = require(cwd + '/' + configfile)
            appdir = path.dirname(configfile);
        process.chdir(appdir);
    } else {
        console.log('Missing or invalid configuration file');
        console.log(usage);
        process.exit(1);
    }
    /**
     * Start the kernel
     */
    var start = function() {
        return new kernel(
             config.config
            ,config.boot
            ,config.run
            ,config.shutdown
        );
    }, krnlobj = start();
    /**
     * If watch changes, then restart the kernel everytime
     * a file changes inside the appdir
     */
    if (typeof watch !== 'undefined' && watch) {
        var md5sum, listen,
        watchintv = setInterval(function() {
            if (!listen) {
                listen = true;
                krnlobj.on(kernel.prototype.OFFLINE, function() {
                    krnlobj = start();
                    listen = false;
                });
            }
            exec('ls -alR . | md5sum | sed s/[^a-zA-Z0-9]//g', function(err, stdout, stderr) {
                if (!err) {
                    if (md5sum && md5sum !== stdout) {
                        krnlobj.shutdown('appdir changed, rebooting');
                    }
                    md5sum = stdout;
                } else {
                    console.log(err, stderr);
                }
            });
        }, 1000);
    }
    /**
     * Term signals
     */
    var shutdown = function() {
        krnlobj.removeAllListeners(kernel.prototype.OFFLINE);
        krnlobj.shutdown('got term signal, going down');
        clearInterval(watchintv);
    };
    process.once('SIGTERM', shutdown);
    process.once('SIGINT',  shutdown);
}