/******************************************************************************
 * Lemonade JS
 * https://lemonadejs.com
 *
 * ----
 *
 * Instance
 *
 *  Creates a context for loading models / controllers / intervals and setting
 *  different properties; it's not shared between requests and it's meant to
 *  clear all used memory by a request on request end
 *
 *  Has some basic abstract methods which instance creators should overload
 *
 *****************************************************************************/

function Instance () {
    this.id        = 'generic-' + Math.floor(Math.random() * 1000);
    this.starttime = new Date();
    this.endtime   = null;
}

/**
 * Instance end
 * @abstract
 */
Instance.prototype.end = function(error, data) {};

/**
 * Instance echo
 * @abstract
 */
Instance.prototype.echo = function(data) {};

/**
 * Instance error
 * @abstract
 */
Instance.prototype.error = function(data) {};

/**
 * Include a controller
 * @param {String} path
 */
Instance.prototype.controller = function(path) {
    return this.kernel.include.call(this,

           this.kernel.appdir + '/'
         + this.kernel.config._KERNEL_DIR_CONTROLLERS + '/'
         + path

    );
};

/**
 * Include an interval
 * @param {String} path
 */
Instance.prototype.interval = function(path) {
    return this.kernel.include.call(this,

          this.kernel.appdir + '/'
        + this.kernel.config._KERNEL_DIR_INTERVALS + '/'
        + path

    );
};

/**
 * Include a model
 * @param {String} path
 */
Instance.prototype.model = function(path) {
    return this.kernel.include.call(this,

          this.kernel.appdir + '/'
        + this.kernel.config._KERNEL_DIR_MODELS + '/'
        + path

    );
};

/**
 * Export
 */
exports = module.exports = Instance;