/**
 * Cassandra loader script
 * @return {Cassandra}
 */

function cassandraLoader () {}

/**
 * Default keyspace
 * @type {string}
 */
cassandraLoader.prototype.defaultcluster = '';

/**
 * Default keyspace
 * @type {string}
 */
cassandraLoader.prototype.defaultkeyspace = '';

/**
 * References to adaptors (a new adaptor is
 * created for each unique keyspace)
 * @type {{}}
 */
cassandraLoader.prototype.adaptors = {};

/**
 * Load
 * @param {array} hosts
 * @param {string} keyspace
 * @param {object} options
 * @param {function} callback
 */
cassandraLoader.prototype.load = function(hosts, keyspace, options, callback) {

    if (!hosts && !keyspace) {
		throw new Error(
			'Cassandra missing config. Define hosts/user/pass/keyspace in the configuration file'
		);
	}

    var nodes = [];

    for (var key in hosts) {
        nodes.push(hosts[key].host + ':' + hosts[key].port);
    }

    this.defaultcluster     = {
         hosts   : nodes
        ,options : options
    };
    this.defaultkeyspace    = keyspace;

	this.link(keyspace);

    callback();
};

/**
 * Return a singleton for the adaptor based on keyspace (there will be a
 * different connection pool and adaptor object for each keyspace name used)
 * @param {string} keyspace
 * @return {object}
 */
cassandraLoader.prototype.link = function(keyspace) {

	keyspace = keyspace || this.defaultkeyspace;

	if (!this.adaptors || !this.adaptors[keyspace]) {

		this.adaptors[keyspace] = this.kernel.include(this.kernel.kerneldir + 'adaptors/cassandra').singleton(
			 keyspace
			,this.defaultcluster
			,keyspace
		);

	}

    return this.adaptors[keyspace];
};

/**
 * Gather up statuses from all open adaptors
 * @return {object}
 */
cassandraLoader.prototype.status = function() {
	var merge = {};
	for (var keyspace in this.adaptors) {
		merge[keyspace] = this.adaptors[keyspace].status;
	}
	return merge;
};

/**
 * Unload all adaptors (prepare for shutdown)
 * @param {function} cb
 */
cassandraLoader.prototype.unload = function(cb) {
	var noloaded = Object.keys(this.adaptors).length;
	if (!noloaded) {
		cb(null, 'pool not created');
	} else {
		for (var keyspace in this.adaptors) {
			this.adaptors[keyspace].shutdown(function() {

                delete this.adaptors[keyspace];

				if (!--noloaded) {
					cb(null, 'pool drained');
				}
			}.bind(this));
		}
	}
};

/**
 * Export
 */
exports = module.exports = cassandraLoader;