/******************************************************************************
 * Lemonade JS
 * https://lemonadejs.com
 *
 * ----
 *
 * Rabbitmq link loader script
 *
 *****************************************************************************/

function rabbitLoader() {}

/**
 * Adaptor reference
 * @type {object}
 */
rabbitLoader.prototype.adaptor;

/**
 * Configuration reference
 * @type {object}
 */
rabbitLoader.prototype.host;

/**
 * Load
 * @param {object} host
 * @param {function} cb
 */
rabbitLoader.prototype.load = function(host, cb) {

    this.adaptor = this.kernel.include(this.kernel.kerneldir + 'adaptors/rabbit').new(host);
    this.host    = host;

    /**
     * Cb when connected
     */
    this.kernel.on(this.adaptor.CONNECTED, function(msg) {
        cb(null, msg || 'connected');
    }.bind(this));
    /**
     * Handle errors
     */
    this.kernel.on(this.adaptor.ERROR, function(err) {
        this.kernel.error(
            new Error(err)
        );
    }.bind(this));
    /**
     * Handle disconnect
     */
    this.kernel.on(this.adaptor.DISCONNECTED, function(err) {
        if (this.unloadcb) {
            this.unloadcb(err);
        }
    }.bind(this));
};

/**
 * Kernel link
 */
rabbitLoader.prototype.link = function() {
    return this.adaptor;
};

/**
 * Status object
 */
rabbitLoader.prototype.status = function() {
    return this.adaptor.status();
};

/**
 * Shutdown
 * @param {function} cb
 */
rabbitLoader.prototype.unload = function(cb) {
    /**
     * After rabbit.connection.end() there is
     * an ECONNRESET emit, avoid it for now
     *
     * TODO : fix this with future client releases
     */
    this.kernel.removeAllListeners(this.adaptor.ERROR);

    if (!this.adaptor) {
        cb(null, 'not connected');
    } else {
        this.unloadcb = cb;
        this.adaptor.shutdown();
    }
};

/**
 * Export
 */
exports = module.exports = rabbitLoader;